from django.conf import settings

from suds.client import Client

if settings.DEBUG:
    import logging
    logging.basicConfig(level=logging.INFO)
    logging.getLogger('suds.client').setLevel(logging.INFO)

PATH = settings.OPENKM['Path']

OPENKM_WSDLS = {
    'Auth': settings.OPENKM['Host'] + '%s/OKMAuth?wsdl' % PATH,
    'Bookmark': settings.OPENKM['Host'] + '%s/OKMBookmark?wsdl' % PATH,
    'Document': settings.OPENKM['Host'] + '%s/OKMDocument?wsdl' % PATH,
    'Search': settings.OPENKM['Host'] + '%s/OKMSearch?wsdl' % PATH,
    'Note': settings.OPENKM['Host'] + '%s/OKMNote?wsdl' % PATH,
    'Folder': settings.OPENKM['Host'] + '%s/OKMFolder?wsdl' % PATH,
    'Property': settings.OPENKM['Host'] + '%s/OKMProperty?wsdl' % PATH,
    'PropertyGroup': settings.OPENKM['Host'] + '%s/OKMPropertyGroup?wsdl' % PATH,
    'Repository': settings.OPENKM['Host'] + '%s/OKMRepository?wsdl' % PATH,
    }

def get_service(class_name):
    return Client(OPENKM_WSDLS[class_name]).service

def get_client(class_name):
    return Client(OPENKM_WSDLS[class_name])

def get_token():
    auth = Auth()
    auth.login()
    return auth.token

class BaseService(object):

    def __init__(self, start_session=True):
        self.service = get_service(self.__class__.__name__)
        self.client = get_client(self.__class__.__name__)
        if start_session:
            self.token = get_token()


class Auth(BaseService):
    """ Methods related to authentication, granting and revoking privileges. """

    def __init__(self):
        super(Auth, self).__init__(start_session=False)

    def login(self, user=settings.OPENKM['User'], password=settings.OPENKM['Password']):
        self.token = self.service.login(user=user, password=password)

    def logout(self):
        return self.service.logout(token=self.token)

    def get_users(self):
        return self.service.getUsers(token=self.token)

    def get_roles(self):
        return self.service.getRoles(token=self.token)

    def grant_user(self):
        pass

    def revoke_user(self):
        pass

    def get_granted_users(self, node_path=None):
        return self.service.getGrantedUsers(token=self.token, nodePath=node_path)

    def grant_role(self, node_path, role, permissions, recursive):
        return self.service.grantRole(token=self.token, nodePath=node_path, role=role,
            permissions=permissions, recursive=recursive)

    def revoke_role(self):
        pass

    def get_granted_roles(self):
        pass


class Document(BaseService):
    """Methods related to document management. """

    def new(self):
        """
        Returns a document object
        """
        return self.client.factory.create('document')

    def create(self, doc, content):
        """
        Create a new document in the repository.
        :param Document object (use self.new())
        :param content Java byte[] compatible (use make_file_java_byte_array_compatible())
        :return A document object with the properties of the new created document.
        """
        return self.service.create(token=self.token, doc=doc, content=content)


    def delete(self, doc_path):
        """
        Removes a document from the repository and move it to the user trash.
        :param doc_path string
        :return none
        """
        return self.service.delete(token=self.token, docPath=doc_path)


    def lock(self, doc_path):
        """
        Lock a document, so only is editable by the locker.
        :param doc_path: string
        :return A Collection of Versions with every document version.
        """
        return self.service.lock(token=self.token, docPath=doc_path)


    def unlock(self, doc_path):
        """
        Unlock a document, so will be editable for other users.
        :param doc_path: string
        :return none
        """
        return self.service.unlock(token=self.token, docPath=doc_path)


    def rename(self, doc_path, new_name):
        """
        Rename a document in the repository.
        :param doc_path string
        :param new_name string
        :return A document object with the new document properties.
        """
        return self.service.rename(token=self.token, docPath=doc_path, newName=new_name)


    def move(self, doc_path, new_name):
        """
        Move a document to another location in the repository.
        :param doc_path string
        :param new_name string
        :return none
        """
        return self.service.move(token=self.token, docPath=doc_path, newName=new_name)


    def get_properties(self, doc_path):
        """
        Obtain document properties from the repository.
        :param doc_path string
        :return The document properties.
        """
        return self.service.getProperties(token=self.token, docPath=doc_path)


    def set_properties(self, doc):
        """
        Set the properties of a repository document
        :param doc Document object
        :return none
        """
        return self.service.setProperties(token=self.token, doc=doc)


    def set_content(self, doc_path, content):
        """
        Set document content in the repository.
        :param doc_path string
        :param content byte array (Java)
        :return none
        """
        return self.service.setContent(token=self.token, docPath=doc_path, content=content)


    def get_content(self, doc_path, checkout):
        """Obtain document content from the repository.
        :param doc_path string
        :param checkout boolean
        """
        return self.service.getContent(token=self.token, docPath=doc_path, checkout=checkout)


    def get_content_by_version(self, doc_path, version_id):
        """
        Obtain document content from the repository.
        :param doc_path
        :param version_id
        """
        return self.service.getContentByVersion(token=self.token, docPath=doc_path, versionId=version_id)


    def checkout(self, doc_path):
        """
        Checkout the document to edit it. The document can't be edited by another user until it is checked in o the checkout is cancelled.
        :param doc_path string
        :return A Collection with the child documents
        """
        return self.service.checkout(token=self.token, docPath=doc_path)


    def cancel_checkout(self, doc_path):
        """
        Cancel a previous checked out state in a document.
        :param doc_path string The path that identifies an unique document.
        :return A Collection with the child documents.
        """
        return self.service.cancelCheckout(token=self.token, docPath=doc_path)


    def checkin(self, doc_path):
        """
        Check in the document to create a new version.
        :param doc_path string
        :return A version object with the properties of the new generated version.
        """
        return self.service.checkin(token=self.token, docPath=doc_path)


    def get_version_history(self, doc_path):
        """
        Get the document version history.
        :param doc_path string
        :return A Collection of Versions with every document version.
        """
        return self.service.getVersionHistory(token=self.token, docPath=doc_path)


    def restore_version(self, doc_path, version_id):
        """
        Revert the document to an specific previous version.
        :param doc_path string
        :param version_id string
        """
        return self.service.restoreVersion(token=self.token, docPath=doc_path, versionId=version_id)

    def get_children(self, folder_path):
        """
        Retrieve a list of child documents from an existing folder.
        :param folder_path string
        :return A Collection with the child documents.
        """
        return self.service.getChilds(token=self.token, fldPath=folder_path)

    def is_valid(self, doc_path):
        """
        Test if a document path is valid.
        :param doc_path
        :return boolean True if the path denotes a document, otherwise false.
        """
        return self.service.isValid(token=self.token, docPath=doc_path)


    def get_path(self, uuid):
        """
        The the document path from a UUID.
        :param uuid string
        :return The document path or null if this UUID does not correspond to a document node.
        """
        return self.service.getPath(token=self.token, uuid=uuid)

class Search(BaseService):
    """Methods related to repository search. """

    def by_content(self, words):
        """Search for documents using it indexed content.  """
        return self.service.findByContent(self.token, words)

    def by_name(self, words):
        """Search for documents by document name. """
        return self.service.findByName(token=self.token, name=words)

    def by_keyword(self, keywords):
        """Search for documents using it associated keywords. """
        return self.service.findByKeywords(token=self.token, keywords=keywords)

    def by_statement(self, statement, type):
        """
        Example (returns all published documents):
        statement = "/jcr:root/okm:root//element(*,okm:document)[okp:published.status='published']"
        """
        return self.service.findByStatement(token=self.token, statement=statement, type=type)

    def find(self, params):
        """ Performs a complex search by content, name and keywords (between others). """
        return self.service.find(token=self.token, params=params)

    def get_keyword_map(self, filter):
        """ Return a Keyword map. This is a hash with the keywords and the occurrence.  """
        return self.service.getKeywordMap(token=self.token, filter=filter)

    def get_categorised_documents(self, category_id):
        """ Get the documents within a category """
        return self.service.getCategorizedDocuments(token=self.token, categoryId=category_id)


class Bookmark(BaseService):
    """Methods related to bookmark management """

    def add(self, node_path, bookmark_name):
        """Add a new bookmark which points to this document """
        return self.service.add(self.token, node_path, bookmark_name)

    def get(self, bookmark_id):
        """Get info from a previously created bookmark. """
        return self.service.get(token=self.token, bmId=bookmark_id)

    def remove(self, bookmark_id):
        """Remove a bookmark. """
        return self.service.remove(token=self.token, bmId=bookmark_id)

    def rename(self, bookmark_id, new_name):
        """Rename a previous stored bookmark. """
        return self.service.rename(token=self.token, bmId=bookmark_id, newName=new_name)

    def get_all(self):
        """Retrieve a users bookmark collection. """
        return self.service.getAll(token=self.token)


class Note(BaseService):
    """Methods related to document notes management. """

    def add(self, node_path, text):
        """Add a note to a document. """
        return self.service.add(token=self.token, nodePath=node_path, text=text)

    def get(self, note_path):
        """Get note from document """
        return self.service.get(token=self.token, notePath=note_path)

    def remove(self, note_path):
        """Remove a note from a document. """
        return self.service.remove(token=self.token, notePath=note_path)

    def set(self, note_path, text):
        """Set a new text to document note. """
        return self.service.set(token=self.token, notePath=note_path, text=text)

    def list(self, node_path):
        """Retrieve a list of notes from a document. """
        return self.service.list(token=self.token, nodePath=node_path)


class Folder(BaseService):

    def new(self):
        return self.client.factory.create('folder')

    def create(self, folder_obj):
        return self.service.create(token=self.token, fld=folder_obj)

    def get_properties(self, folder_path):
        return self.service.getProperties(token=self.token, fldPath=folder_path)

    def delete(self, folder_path):
        return self.service.delete(token=self.token, fldPath=folder_path)

    def delete_children(self, folder_path):
        children = self.get_children(folder_path)
        for child in children.item:
            self.delete(child.path)

    def rename(self, folder_path, new_folder_path):
        return self.service.rename(token=self.token, fldPath=folder_path, newName=new_folder_path)

    def move(self, current_folder_path, destination_path):
        return self.service.move(token=self.token, fldPath=current_folder_path, dstPath=destination_path)

    def get_children(self, folder_path):
        """Remove a note from a document. """
        return self.service.getChilds(token=self.token, fldPath=folder_path)

    def is_valid(self, folder_path):
        return self.service.isValid(token=self.token, fldPath=folder_path)

    def get_path(self, uuid):
        return self.service.getPath(token=self.token, uuid=uuid)


class Property(BaseService):

    def add_category(self, node_path, category_uuid):
        return self.service.addCategory(self.token, nodePath=node_path, catId=category_uuid)

    def remove_category(self, node_path, category_uuid):
        return self.service.removeCategory(token=self.token, nodePath=node_path, catId=category_uuid)

    def add_keyword(self, node_path, keyword):
        return self.service.addKeyword(token=self.token, nodePath=node_path, keyword=keyword)

    def remove_keyword(self, node_path, keyword):
        ''' Add a keyword to a document.  '''
        return self.service.removeKeyword(token=self.token, nodePath=node_path, keyword=keyword)


class PropertyGroup(BaseService):
    '''
    Methods related to Property Groups.
    '''

    def add_group(self, node_path, group_name):
        ''' Add a property group to a document. '''
        return self.service.addGroup(token=self.token, nodePath=node_path, grpName=group_name)

    def remove_group(self, node_path, group_name):
        return self.service.removeGroup(token=self.token, nodePath=node_path, grpName=group_name)

    def get_groups(self, node_path):
        ''' Get groups assigned to a document. '''
        return self.service.getGroups(token=self.token, nodePath=node_path)

    def get_all_groups(self):
        ''' Get all groups defined in the system. '''
        return self.service.getAllGroups(token=self.token)

    def get_properties(self, node_path, group_name):
        return self.service.getProperties(token=self.token, nodePath=node_path, grpName=group_name)

    def set_properties(self, node_path, group_name, properties):
        return self.service.setProperties(token=self.token, nodePath=node_path, grpName=group_name, properties=properties)

    def has_group(self, node_path, group_name):
        return self.service.hasGroup(token=self.token, nodePath=node_path, grpName=group_name)


class Repository(BaseService):

    def get_root_folder(self):
        return self.service.getRootFolder(self.token)

    def get_trash_folder(self):
        return self.service.getTrashFolder(token=self.token)

    def get_templates_folder(self):
        return self.service.getTemplatesFolder(token=self.token)

    def get_personal_folder(self):
        return self.service.getPersonalFolder(token=self.token)

    def get_mail_folder(self):
        return self.service.getMailFolder(token=self.token)

    def get_thesaurus_folder(self):
        return self.service.getThesaurusFolder(token=self.token)

    def get_categories_folder(self):
        return self.service.getCategoriesFolder(token=self.token)

    def purge_trash(self):
        return self.service.purgeTrash(token=self.token)

    def has_node(self, path):
        """ Test if a node path exists """
        return self.service.hasNode(token=self.token, path=path)

    def get_path(self, uuid):
        """ Obtain the node path with a given uuid. """
        return self.service.getPath(token=self.token, uuid=uuid)




from suds import WebFault
from django.conf import settings

from .client import Auth, Document, Property, PropertyGroup, Repository
from .facades import Category, Keyword, DirectoryListing
from .models import OpenKmFolderList
from .utils import find_key

class SyncPropertiesException(Exception):
    pass

class SyncProperties(object):

    PROPERTY_GROUP = 'okg:customProperties'

    # django model attributes -> openkm properties
    MODEL_PROPERTY_MAP = {
        'name': 'Title',
        'description': 'Description',
        }

    def __init__(self):
        auth = Auth()
        auth.login()
        self.property_group = PropertyGroup()

    def document_has_property_group(self, document, group_name):
        return self.property_group.has_group(document.path, group_name)

    def update_properties(self, resource, properties):
        for property in properties.item:
            if property.label in self.MODEL_PROPERTY_MAP.values():
                django_attr = find_key(self.MODEL_PROPERTY_MAP, property.label)
                property.value = resource.__getattribute__(django_attr)
        return properties

    def update_django_model_attributes(self, model, properties):
        for property in properties.item:
            if property.label in self.MODEL_PROPERTY_MAP.values():
                model_attribute_name = find_key(self.MODEL_PROPERTY_MAP, property.label)
                setattr(model, model_attribute_name, property.value)
        return model.save()

    def django_to_openkm(self, resource, document):
        # if document has the group, then grab the properties and remove it, as
        # we need to re-add it if we want to modify it
        if self.document_has_property_group(document, self.PROPERTY_GROUP):
            properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)
            self.property_group.remove_group(document.path, self.PROPERTY_GROUP)

        self.property_group.add_group(document.path, self.PROPERTY_GROUP)

        if not properties:
            properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)

        updated_properties = self.update_properties(resource, properties)
        self.property_group.set_properties(document.path, self.PROPERTY_GROUP, updated_properties)

    def openkm_to_django(self, resource, document):
        if not self.document_has_property_group(document, self.PROPERTY_GROUP):
            raise SyncPropertiesException('OpenKM document does not have %s' % self.PROPERTY_GROUP)
        properties = self.property_group.get_properties(document.path, self.PROPERTY_GROUP)



class SyncKeywords(object):

    def __init__(self):
        self.keyword = Keyword()

    def get_tags_from_resource(self, resource):
        """
        Returns an object's tags as a list
        """
        return resource.tag_set.split(',')

    def add_keyword_to_openkm_document(self, path, keyword):
        return self.keyword.add(path, keyword.strip())

    def write_keywords_to_openkm_document(self, path, keywords):
        """
        :param path: string  The document path on OpenKM
        :param keywords: list The keywords to be associated with a document
        """
        for keyword in keywords:
            self.add_keyword_to_openkm_document(path, keyword)

    def confirm_keywords_written_to_openkm(self, path, expected_keywords):
        """
        Query OpenKM to make sure tags have been set
        :param path: OpenKM node path of document
        :param expected_keywords: list
        """
        openkm_keywords = self.keyword.get_for_document(path)
        diff = set(expected_keywords).difference(set(openkm_keywords))

        if diff == 0:
            return True
        else:
            return False

    def single_document_django_to_helix(self, document, openkm_document):
        """
        Writes the tags from a GSA resource to a Helix document as keywords
        Assumes that the document exists on OpenKM
        :param document: Document model object
        :param openkm_document: OpenKM document object
        """
        tags = self.get_tags_from_resource(document)
        self.write_keywords_to_openkm_document(openkm_document.path, tags)

    def single_document_helix_to_django(self, document, openkm_document):
        """
        Updates the tags on a Django model from the keywords on OpenKM
        :param document: Resource model object
        :param openkm_document: OpenKM document object
        """
        keywords = self.keyword.get_for_document(openkm_document.path)
        tags = ','.join(keywords)
        document.update_tags(tags)

class SyncCategories(object):
    """
    Syncronises categories from Django to OpenKM.
    """
    # Django Model Class -> OpenKM Category
    MODEL_CATEGORY_MAP = {}

    def __init__(self):
        self.category = Category()

    def create_top_level_categories(self, parent_path=False):
        """
        Creates the top level categories, as first level children of the parent_path
        :param parent_path: (string) defaults to category root
        """
        if not parent_path:
            parent_path = self.category.get_category_root().path

        for category_name in self.MODEL_CATEGORY_MAP.values():
            path = self.category.construct_valid_path_string(parent_path, category_name)
            try:
                self.category.create(path)
                logger.info("%s created" % path)
            except:
                logger.info("%s creation failed" % path)

    def get_child_categories(self, parent_path):
        """
        :param parent_path: string node path
        :return object
        """
        if not parent_path:
            parent_path = self.category.get_category_root().path

        return self.category.folder.get_children(parent_path)

    def django_to_openkm(self):
        for django_model_class, openkm_model_name in self.MODEL_CATEGORY_MAP.items():
            self.create_categories_from_django_model(django_model_class, openkm_model_name)

    def create_categories_from_django_model(self, model_class, category_name):
        """
        Creates child sub-categories on OpenKM in a parent folder.
        The parent category should already exist.
        :param model_class: model class object
        :param category_name: string (name of the Category on OpenKM)
        """
        objects = model_class.objects.all()

        for object in objects:
            try:
                self.category.create('/okm:categories/%s/%s' % (category_name,\
                                                                object.__unicode__().replace('/','')))
            except:
                logger.info("%s creation failed" % object.__unicode__().replace('/',''))

    def openkm_to_django(self):
        pass

    def get_many_to_many_fields_from_model(self, document, related_model_class):
        """
        Accepts a Document and a single class object of a many-to-many field
        :returns queryset the related model objects associated with the given resource
        """
        method_name = '%s_set' % related_model_class.__name__.lower()
        _set = getattr(document, method_name)

        return _set.all()

    def get_related_objects_from_model(self, document, related_model_name):
        """
        :param document: object instance of Document
        :param related_model_name: string relate model name
        :returns a list of related model objects
        Assumes that the related model is:
        - many-to-many
        - is accessible from the resource via modelname_set()
        For example if called with a resource and the string region
        it will return the regions associated with this resource
        """
        method_name = '%s_set' % related_model_name
        if hasattr(document, method_name):
            _member = getattr(document, method_name)
            return _member.all()
        else:
            logger.error("Object does not have method region_set")


class SyncResourceException(Exception):
    pass

class SyncResource(object):
    """
    Syncs a Resource object to OpenKM
    """
    def __init__(self, document):
        if not isinstance(document, Document):
            raise SyncResourceException('Must pass a Resource instance to be synced')
        self.auth = Auth()
        self.auth.login()
        self.document = Document()
        self.repository = Repository()
        self.document = document
        self.sync_keywords = SyncKeywords()
        self.sync_categories = SyncCategories()

    def new_openkm_document_object(self):
        """ Returns a local object instance """
        return self.document.new()

    def document_exists_on_openkm(self, path):
        return self.repository.has_node(path)

    def build_openkm_path(self, base_path, file_name):
        return "%s%s" % (base_path, file_name)

    def upload_to_openkm(self, file):
        """
        Uploads the document to the OpenKM server
        :param file: a file object
        """
        path = self.build_openkm_path(settings.OPENKM['UploadRoot'], file.name)
        openkm_document = self.new_openkm_document_object()
        openkm_document.path = settings.OPENKM['UploadRoot'] + file.name
        content = utils.make_file_java_byte_array_compatible(file)

        try:
            return self.document.create(openkm_document, content)
        except Exception, e:
            logger.error("There was a problem uploading the document: %s" % file.name)

    def keywords(self):
        """
        TAGS -> KEYWORDS
        Writes the tags to OpenKM as keywords and confirms that they have been added
        :returns boolean:  True on success, False on fail
        """
        tags = self.sync_keywords.get_tags_from_resource(self.document)
        self.sync_keywords.write_keywords_to_openkm_document(self.path, tags)
        return self.sync_keywords.confirm_keywords_written_to_openkm(self.path, tags)

    def categories(self):
        for related_model_class in self.sync_categories.MODEL_CATEGORY_MAP.keys():
            self.sync_categories.get_many_to_many_fields_from_model(self.document, related_model_class)

    def properties(self):
        pass

    def file(self):
        pass


class SyncFolderList(object):
    """
    Local storage of OpenKM folder metadata
    @todo make this generic to pickup all the folders across OpenKM, not just
    categories
    """
    def __init__(self):
        self.category = Category()
        self.dir = DirectoryListing()
        self.repository = Repository()

    def execute(self):
        paths = self.get_list_of_root_paths()
        folders = self.traverse_folders(paths)
        self.save(folders)

    def get_list_of_root_paths(self):
        return [self.category.get_category_root().path, self.repository.get_root_folder().path]

    def traverse_folders(self, paths):
        folders = []
        for path in paths:
            folders.extend(self.dir.traverse_folders(path))

        return folders

    def save(self, folders):
        for folder in folders:
            try:
                cl, created = OpenKmFolderList.objects.get_or_create(uuid=folder.uuid)
                cl.author = folder.author
                cl.created = folder.created
                cl.has_childs = folder.hasChilds
                cl.path = folder.path
                cl.permissions = folder.permissions
                cl.subscribed = folder.subscribed
                cl.save()
            except Exception, e:
                logger.error(e)



from django.test import TestCase
from django.conf import settings

from .client import Auth, Document, Folder, Note, OPENKM_WSDLS, Repository, get_token
from .facades import Keyword, Category, DirectoryListing
from .sync import SyncKeywords, SyncCategories, SyncResource, SyncFolderList

#from tce.resources.models import Resource


class ClientTest(TestCase):
    """ Tests of functions and settings """
    
    def setUp(self):
        pass
    
    def test_settings(self):
        """ Check that the config values are set in the settings file """
        keys = ('Logging', 'UploadRoot', 'Host', 'User', 'Password')
        for key in keys:
            self.assertTrue(settings.OPENKM.has_key(key))
            
    def test_wsdl_map(self):
        """ Check fof the presence of the WSDL dict map """
        self.assertTrue(isinstance(OPENKM_WSDLS, dict))


class FolderTest(TestCase):
    
    def setUp(self):
        self.folder = Folder()

    def test_get_children(self):
        children = self.folder.get_children('/okm:root/')
        self.assertTrue(hasattr(children, 'item'))

class AuthTest(TestCase):
    
    def setUp(self):
        self.auth = Auth()
        
    def test_login(self):
        """ Check we can login and get a token """
        self.auth.login()
        self.assertTrue(hasattr(self.auth, 'token'), msg="Token has not been set")
        self.assertTrue(len(self.auth.token) > 1, msg="Token is empty")
        
    def test_logout(self):
        """ A successful logout should destroy the session token """
        self.auth.login()
        has_token = hasattr(object, 'token')
        self.auth.logout()

        try:
            has_token = hasattr(object, 'token')
        except AttributeError:
            has_token = False
        finally:
            self.assertFalse(has_token)

    def has_token_attribute(self, object):
        return hasattr(object, 'token')

    def test_get_roles(self):
        self.auth.login()
        roles = self.auth.get_roles()
        self.assertTrue(hasattr(roles, 'item'), msg="instance expected to contain item[]")
        self.assertTrue(isinstance(roles.item, list), msg="role.item expected to be a list")
    
    def test_grant_role(self):
        pass
    
    def test_revoke_role(self):
        pass
    
    def test_get_users(self):
        pass
    
    def test_grant_user(self):
        pass
    
    def test_revoke_user(self):
        pass
    
    def test_get_granted_users(self):
        pass
    

class DocumentTest(TestCase):
    
    test_doc_path = '%s%s' % (settings.OPENKM['UploadRoot'], 'testing123.pdf')
    
    def setUp(self):
        """
        Login to OpenKM, get the session token and use it to instantiate
        a Document object
        """
        self.test_doc = 'xxxx'
        try:
            self.auth = Auth()
            self.auth.login()

            self.doc = Document()
            
            # create a test document
            test_doc = self.doc.new()
            test_doc.path = self.test_doc_path
            self.doc.create(test_doc, "hf7438hf7843h378ot4837ht7")
            self.test_doc = self.doc.get_properties(self.test_doc_path)
        except AssertionError, detail:
            print detail
        
    def test_token_has_been_set(self):
        """
        Check that the token has been set in both Auth and Document,
        otherwise we won't be doing anything
        """
        self.assertTrue(hasattr(self.auth, 'token'), msg="Token not set in Auth")
        self.assertTrue(hasattr(self.doc, 'token'), msg="Token not set in Document")
        
    def document_object(self, doc):
        """ 
        Test the structure of a document instance 
        """
        keys = ('actualVersion', 'author', 'checkedOut', 'convertibleToDxf',
                'convertibleToPdf', 'convertibleToSwf', 'created', 'language', 'lastModified',
                'locked', 'mimeType', 'path', 'permissions', 'subscribed', 'uuid')
        for key in keys:
            msg = "%s doesn't exist" % key
            self.assertTrue(hasattr(doc, key), msg)

        return True
            
    def test_create(self):
        """ Create a test document """
        test_doc = self.doc.new()
        filename = 'abc.pdf'
        test_doc.path = '%s%s' % (settings.OPENKM['UploadRoot'], filename)
        content = "hf8478y7ro48y7y7t4y78o4"
        self.doc.create(test_doc, content)
        new_document = self.doc.get_properties(test_doc.path)
        
        self.assertEqual(test_doc.path, new_document.path, msg="Created document path does not match that uploaded")
        
        # clean up the file, otherwise it will remain on OpenKM causing the next test run to fail
        self.doc.delete(new_document.path)
        
    def test_lock(self):
        """ Lock and unlock a document """
        self.doc.lock(self.test_doc.path)
        properties = self.doc.get_properties(self.test_doc_path)
        self.assertTrue(properties.locked, msg="Document is not locked")
        
        self.doc.unlock(self.test_doc.path)
        properties = self.doc.get_properties(self.test_doc_path)
        self.assertFalse(properties.locked, msg="Document is locked")
        
    def test_get_properties(self):
        """ Get the properites of a document and check the attributes """
        d = self.doc.get_properties(self.test_doc.path)
        contains_keys = self.document_object(d)
        self.assertTrue(contains_keys) # make sure it contains the expected attributes

    def tearDown(self):
        self.doc.delete(self.test_doc_path)
        self.auth.logout()

def get_test_document_path():
    return '%snotes.pdf' % settings.OPENKM['UploadRoot']

def create_test_document_on_openkm():
    document = Document()
    test_doc = document.new()
    test_doc.path = get_test_document_path()
    document.create(test_doc, "hf7438hf7843h378ot4837ht7")
    return document.get_properties(test_doc.path)

def delete_test_document_on_openkm():
    d = Document()
    d.delete(get_test_document_path())

class NoteTest(TestCase):
    
    def setUp(self):
        self.document = Document()
        self.note = Note()
        self.auth = Auth()
        self.auth.login()
        self.test_document = create_test_document_on_openkm()

    def test_add_and_remove(self):
        # add a note
        text = 'hello, this is a note'
        meta = self.note.add(self.test_document.path, text)
        
        # remove it
        self.note.remove(meta.path)
        
    def test_get_list(self):
        """
        Add some notes to a document and retrieve them
        """
        # add three notes
        notes = ('one', 'two', 'three')
        for note in notes:
            self.note.add(self.test_document.path, note)
        
        # test that the notes returned match those added
        list = self.note.list('/okm:root/gsa/notes.pdf')
        for entered, note in zip(notes, list.item):
            self.assertEqual(entered, note.text)
            self.note.remove(note.path)
            
    def test_get(self):
        text = 'Knock, knock...'
        note = self.note.add(self.test_document.path, text)
        note_meta = self.note.get(note.path)
        self.assertEqual(note_meta.text, text, msg='Returned note does not match')
        self.note.remove(note_meta.path)
        
    def tearDown(self):
        delete_test_document_on_openkm()

KEYWORDS = ('One', 'Two', 'Three', 'Hammertime!')

class KeywordTest(TestCase):

    def setUp(self):
        self.keyword = Keyword()
        self.test_document = create_test_document_on_openkm()

    def test_add(self):
        for keyword in KEYWORDS:
            self.keyword.add(self.test_document.path, keyword)

    def test_remove(self):
        for keyword in KEYWORDS:
            self.keyword.remove(self.test_document.path, keyword)
            
    def tearDown(self):
        delete_test_document_on_openkm()

class SyncKeywordsTest(TestCase):

    def setUp(self):
        self.resource = Resource()
        self.resource.name = 'This is a resource'
        self.resource.description = 'And its description'
        self.resource.tag_set = u'One, Two, Three'
        self.sync_keywords = SyncKeywords()
        self.path = '/okm:root/test-global.html'
        self.tags = self.sync_keywords.get_tags_from_resource(self.resource)

    def test_get_tags_from_resource(self):
        """
        Should take a a comma separated string of tags and return a list
        """
        self.assertTrue(isinstance(self.tags, list), msg="Tags should be returned as a list")

    def test_tags_are_unicode(self):
        tags = self.sync_keywords.get_tags_from_resource(self.resource)
        for tag in tags:
            self.assertTrue(isinstance(tag, unicode), msg="%s is not a string, it is %s" % (tag, type(tag)))

    def test_add_keyword_to_openkm_document(self):
        self.sync_keywords.add_keyword_to_openkm_document(self.path, 'Example')

        # cleanup
        self.sync_keywords.keyword.remove(self.path, 'Example')

    def test_write_keywords_to_openkm_document(self):
        self.sync_keywords.write_keywords_to_openkm_document(self.path, self.tags)

    def test_single_document_django_to_helix(self):
        pass


class CategoryTest(TestCase):

    def setUp(self):
        self.category = Category()
        self.repository = Repository()

    def test_get_category_root_object_structure(self):
        category_root = self.category.get_category_root()
        attrs = ('author', 'created', 'hasChilds', 'path', 'permissions', 'subscribed', 'uuid')
        for attr in attrs:
            self.assertTrue(hasattr(category_root, attr), msg="%s is not an attribute of the object returned \
            by category root" % attr)

    def test_category_root_path(self):
        category_root = self.category.get_category_root()
        expected = "/okm:categories"
        self.assertEquals(category_root.path, expected, msg="Category root returned %s was not %s" \
        % (category_root.path, expected))

    def test_create_and_remove(self):
        """
        Test these together so we clean up after ourselves
        """
        base_path = self.category.get_category_root().path
        new_category_name = 'UnitTest'
        new_category_path = self.category.construct_valid_path_string(base_path, new_category_name)

        # create the category
        expected_path = new_category_path
        new_category = self.category.create(new_category_path)
        self.assertEquals(new_category.path, expected_path)

        # remove the path
        self.category.remove(new_category.path)

        # test that the category has been removed
        self.assertFalse(self.repository.has_node(new_category.path), msg="Category has not been removed \
        node %s found" % new_category.path)

    def test_construct_valid_path_string(self):
        base_path = '/okm:categories'
        category = 'Products'
        expected = '/okm:categories/Products'
        returned = self.category.construct_valid_path_string(base_path, category)
        self.assertEquals(expected, returned, msg="Returned path %s did not match the expected path %s" %\
                                                  (returned, expected))


class MockResource(object):
    """
    Creates a mock resource, with populated many-to-many fields to be used in tests
    """
    def __init__(self):
        self.sync_categories = SyncCategories()

    def generate_test_resource(self):
        """
        Creates a Resource to be used in testing and populates the many-to-many fields
        with query sets.  Currently simple calling .all() on the related models
        """
        resource = Resource(name='test document')
        resource.save()

        # for each model
        for klass in self.sync_categories.MODEL_CATEGORY_MAP.keys():

            # get the model_set many-to-many function of the Resource object
            method_name = '%s_set' % klass.__name__.lower()
            _set = getattr(resource, method_name)

            # populate the resource with objects from the related model
            objects = klass.objects.all()
            for object in objects:
                _set.add(object)

        return resource

    def populate_test_model(self, model_klass, related_model_klasses):
        """
        Populates a model to be used in testing and populates the many-to-many fields
        with query sets.  Currently simple calling .all() on the related models.
        Assumes that the naming convention for related model function names if
        [model_name_lowercase]_set (this comes from a legacy project)
        :param model_klass: the main model class
        :param related_model_klasses: an iterable of related model classes
        """
        main_model = model_klass()
        main_model.save()

        # for each model
        for klass in related_model_klasses:

            # get the many-to-many function of the model_klass object
            method_name = '%s_set' % klass.__name__.lower()
            _set = getattr(main_model, method_name)

            # populate the model_klass instance with objects from the related model
            objects = klass.objects.all()
            for object in objects:
                _set.add(object)

        return main_model



class SyncCategoriesTest(TestCase):

    fixtures = ['resources', 'regions', 'solutions', 'industry', 'roles', 'product', 'methodology']

    def setUp(self):
        self.sync_categories = SyncCategories()
        self.category = Category()
        m = MockResource()
        self.r = m.generate_test_resource()

    def test_category_map(self):
        """ Check for existence only as content may change """
        self.assertTrue(hasattr(self.sync_categories, 'MODEL_CATEGORY_MAP'), msg="Category map dict not found")

    def test_get_child_categories(self):
        """
        Returns a list of folder objects which are the child categories of the parent given as param
        """

        # add a category to root so we will return results in the next step
        category_root_path = self.category.get_category_root().path
        mock_category_path = self.category.construct_valid_path_string(category_root_path, 'DummyXYZ')
        mock_category = self.category.create(mock_category_path)

        child_categories = self.sync_categories.get_child_categories(category_root_path)

        has_child = False
        for child in child_categories.item:
            if child.path == mock_category.path:
                has_child = True
        self.assertTrue(has_child, msg="The mock category %s was not found in the returned \
        child categories of %s" % (mock_category.path, category_root_path))

        # clean up
        self.category.remove(mock_category.path)

    def test_create_categories_from_django_model(self):
        #print self.sync_categories.create_categories_from_django_model(Resource, 'Test')
        pass

    def test_django_to_openkm(self):
        for django_model, openkm_model in self.sync_categories.MODEL_CATEGORY_MAP.items():
            print django_model, openkm_model

    def test_openkm_to_django(self):
        pass

    def test_get_categories_from_gsa(self):
        for model in self.sync_categories.MODEL_CATEGORY_MAP.keys():
            model_name = model.__name__.lower()
            print self.sync_categories.get_related_objects_from_model(self.r, model_name)



class SyncResourceTest(TestCase):

    def setUp(self):
        m = MockResource()
        self.r = m.generate_test_resource()
        self.sr = SyncResource(self.r)
        self.test_document = create_test_document_on_openkm()

    def test_init_was_successful(self):
        """ Check that the objects we need were instantiated correctly """
        self.assertTrue(isinstance(self.sr.document, Resource))
        self.assertTrue(isinstance(self.sr.sync_keywords, SyncKeywords))
        self.assertTrue(isinstance(self.sr.auth, Auth))
        self.assertTrue(isinstance(self.sr.document, Document))
        self.assertTrue(isinstance(self.sr.repository, Repository))

    def test_new_openkm_document_object(self):
        """
        Should return a new openkm document object.  Note this does not yet exist on OpenKM until
        Document.create()
        """
        doc_obj = self.sr.new_openkm_document_object()
        self.assertEquals(str(doc_obj.__class__), 'suds.sudsobject.document')

    def test_document_exists_on_openkm(self):
        path = self.test_document.path
        self.assertTrue(self.sr.document_exists_on_openkm(path), msg="Document was not found on OpenKM: %s" % path)

    def categories(self):
        pass

    def properties(self):
        pass

    def file(self):
        """ Uploads the file
        """
        pass

    def tearDown(self):
        delete_test_document_on_openkm()

class DirectoryListingTest(TestCase):

    def setUp(self):
        self.dir = DirectoryListing()

    def test_traverse(self):
        self.dir.traverse_folders('/okm:categories/')


class SyncFolderListTest(TestCase):

    def setUp(self):
        self.folder_list = SyncFolderList()

    def test_get_list_of_root_paths(self):
        paths = self.folder_list.get_list_of_root_paths()
        self.assertTrue(isinstance(paths, list), msg="Expected return value to be a list")






# Use 'final' as the 4th element to indicate
# a full release

VERSION = (0, 3, 2, 'alpha', 0)
    
def get_short_version():
    return '%s.%s' % (VERSION[0], VERSION[1])

def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    if VERSION[2]:
        # Append 3rd digit if > 0
        version = '%s.%s' % (version, VERSION[2])
    if VERSION[3:] == ('alpha', 0):
        version = '%s pre-alpha' % version
    elif VERSION[3] != 'final':
        version = '%s %s %s' % (version, VERSION[3], VERSION[4])
    return version



from django.db import models

from .facades import Session, DocumentManager, FileSystem

class OpenKmMetadata(models.Model):
    """
    An abstract class which contains the template to store OpenKM metadata
    """
    author = models.CharField(max_length=255, blank=True, null=True)
    created = models.DateTimeField(blank=True, null=True)
    path = models.CharField(max_length=255, blank=True, null=True)
    permissions = models.CharField(max_length=255, blank=True, null=True)
    subscribed = models.CharField(max_length=255, blank=True, null=True)
    uuid = models.CharField(max_length=255, blank=True, null=True)

    class Meta:
        abstract = True

class OpenKmFolderList(OpenKmMetadata):
    """
    Stores category folder metadata from OpenKM in a local table
    """
    has_childs = models.CharField(max_length=255, blank=True, null=True)

    def __unicode__(self):
        return self.path

    class Meta:
        verbose_name = 'OpenKM Folder List'
        verbose_name_plural = verbose_name

class OpenKmDocument(OpenKmMetadata):

    filename = models.CharField(max_length=255, blank=True, null=True)
    file = models.FileField(max_length=255, upload_to='resources/%Y/%m/%d/', blank=True, null=True, help_text="Upload a file from your local machine")

    def save(self, *args, **kwargs):
        """
        Custom save functionality.
        (1) Upload document to OpenKM
        (2) Set the local model fields with the returned metadata from (1)
        *Note that locally stored files will be periodically deleted
        """
        if self.file and self.id is None:
            """ A new resource to be uploaded OpenKM """
            session = Session()
            session.open()
            file_obj = self.file._get_file()
            openkm_document = self.upload_to_openkm(file_obj)
            session.close()
            self.set_model_fields(openkm_document)
            super(OpenKmDocument, self).save(*args, **kwargs)
            return

        super(OpenKmDocument, self).save(*args, **kwargs)

    def upload_to_openkm(self, file_obj):
        """Uploads the document to the OpenKM server """
        document_manager = DocumentManager()
        return document_manager.create(file_obj)

    def set_model_fields(self, openkm_document):
        """
        Set the model's fields values with the meta data returned
        by OpenKM to identify the resource
        """
        self.author = openkm_document.author
        self.created = openkm_document.created
        self.path = openkm_document.path
        self.permissions = openkm_document.permissions
        self.subscribed = openkm_document.subscribed
        self.uuid = openkm_document.uuid

        file_system = FileSystem()
        self.filename = file_system.get_filename_from_path(self.path)

    def __unicode__(self):
        return self.filename

    class Meta:
        verbose_name = 'OpenKM Document'
        verbose_name_plural = 'OpenKM Documents'



